@@ -33,6 +33,8 @@ import hidapi as _hid
 from . import hidpp10 as _hidpp10
 from . import hidpp20 as _hidpp20
 from .base_usb import ALL as _RECEIVER_USB_IDS
+from .base_usb import DEVICES as _DEVICE_IDS
+from .base_usb import other_device_check as _other_device_check
 from .common import KwException as _KwException
 from .common import pack as _pack
 from .common import strhex as _strhex
@@ -88,16 +90,46 @@ class DeviceUnreachable(_KwException):
 #
 
 
+def match(record, bus_id, vendor_id, product_id):
+    return ((record.get('bus_id') is None or record.get('bus_id') == bus_id)
+            and (record.get('vendor_id') is None or record.get('vendor_id') == vendor_id)
+            and (record.get('product_id') is None or record.get('product_id') == product_id))
+
+
+def filter_receivers(bus_id, vendor_id, product_id):
+    """Check that this product is a Logitech receiver and if so return the receiver record for further checking"""
+    for record in _RECEIVER_USB_IDS:  # known receivers
+        if match(record, bus_id, vendor_id, product_id):
+            return record
+
+
 def receivers():
-    """List all the Linux devices exposed by the UR attached to the machine."""
-    for receiver_usb_id in _RECEIVER_USB_IDS:
-        for d in _hid.enumerate(receiver_usb_id):
-            yield d
+    """Enumerate all the receivers attached to the machine."""
+    for dev in _hid.enumerate(filter_receivers):
+        yield dev
+
+
+def filter_devices(bus_id, vendor_id, product_id):
+    """Check that this product is of interest and if so return the device record for further checking"""
+    for record in _DEVICE_IDS:  # known devices
+        if match(record, bus_id, vendor_id, product_id):
+            return record
+    return _other_device_check(bus_id, vendor_id, product_id)  # USB and BT devices unknown to Solaar
+
+
+def wired_devices():
+    """Enumerate all the USB-connected and Bluetooth devices attached to the machine."""
+    for dev in _hid.enumerate(filter_devices):
+        yield dev
+
+
+def filter_either(bus_id, vendor_id, product_id):
+    return filter_receivers(bus_id, vendor_id, product_id) or filter_devices(bus_id, vendor_id, product_id)
 
 
 def notify_on_receivers_glib(callback):
     """Watch for matching devices and notifies the callback on the GLib thread."""
-    _hid.monitor_glib(callback, *_RECEIVER_USB_IDS)
+    return _hid.monitor_glib(callback, filter_either)
 
 
 #
@@ -149,7 +181,7 @@ def close(handle):
     return False
 
 
-def write(handle, devnumber, data):
+def write(handle, devnumber, data, long_message=False):
     """Writes some data to the receiver, addressed to a certain device.
 
     :param handle: an open UR handle.
@@ -166,7 +198,7 @@ def write(handle, devnumber, data):
     assert data is not None
     assert isinstance(data, bytes), (repr(data), type(data))
 
-    if len(data) > _SHORT_MESSAGE_SIZE - 2 or data[:1] == b'\x82':
+    if long_message or len(data) > _SHORT_MESSAGE_SIZE - 2 or data[:1] == b'\x82':
         wdata = _pack('!BB18s', 0x11, devnumber, data)
     else:
         wdata = _pack('!BB5s', 0x10, devnumber, data)
@@ -225,7 +257,7 @@ def _read(handle, timeout):
         timeout = int(timeout * 1000)
         data = _hid.read(int(handle), _MAX_READ_SIZE, timeout)
     except Exception as reason:
-        _log.error('read failed, assuming handle %r no longer available', handle)
+        _log.warn('read failed, assuming handle %r no longer available', handle)
         close(handle)
         raise NoReceiver(reason=reason)
 
@@ -313,7 +345,7 @@ del namedtuple
 
 
 # a very few requests (e.g., host switching) do not expect a reply, but use no_reply=True with extreme caution
-def request(handle, devnumber, request_id, *params, no_reply=False):
+def request(handle, devnumber, request_id, *params, no_reply=False, return_error=False, long_message=False):
     """Makes a feature call to a device and waits for a matching reply.
     :param handle: an open UR handle.
     :param devnumber: attached device number.
@@ -350,7 +382,7 @@ def request(handle, devnumber, request_id, *params, no_reply=False):
     ihandle = int(handle)
     notifications_hook = getattr(handle, 'notifications_hook', None)
     _skip_incoming(handle, ihandle, notifications_hook)
-    write(ihandle, devnumber, request_data)
+    write(ihandle, devnumber, request_data, long_message)
 
     if no_reply:
         return None
@@ -381,8 +413,7 @@ def request(handle, devnumber, request_id, *params, no_reply=False):
                             '(%s) device 0x%02X error on request {%04X}: %d = %s', handle, devnumber, request_id, error,
                             _hidpp10.ERROR[error]
                         )
-                    return
-
+                    return _hidpp10.ERROR[error] if return_error else None
                 if reply_data[:1] == b'\xFF' and reply_data[1:3] == request_data[:2]:
                     # a HID++ 2.0 feature call returned with an error
                     error = ord(reply_data[3:4])
@@ -437,7 +468,7 @@ def request(handle, devnumber, request_id, *params, no_reply=False):
     # raise DeviceUnreachable(number=devnumber, request=request_id)
 
 
-def ping(handle, devnumber):
+def ping(handle, devnumber, long_message=False):
     """Check if a device is connected to the receiver.
 
     :returns: The HID protocol supported by the device, as a floating point number, if the device is active.
@@ -449,7 +480,7 @@ def ping(handle, devnumber):
     # print ('\n  '.join(str(s) for s in _inspect.stack()))
 
     assert devnumber != 0xFF
-    assert devnumber > 0x00
+    assert devnumber >= 0x00
     assert devnumber < 0x0F
 
     # randomize the SoftwareId and mark byte to be able to identify the ping
@@ -461,7 +492,7 @@ def ping(handle, devnumber):
     ihandle = int(handle)
     notifications_hook = getattr(handle, 'notifications_hook', None)
     _skip_incoming(handle, ihandle, notifications_hook)
-    write(ihandle, devnumber, request_data)
+    write(ihandle, devnumber, request_data, long_message)
 
     # we consider timeout from this point
     request_started = _timestamp()
